SystemOrganization addCategory: #'FileSystem-Memory'!

FileSystemHandle subclass: #MemoryHandle
	instanceVariableNames: 'entry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryHandle commentStamp: '<historical>' prior: 0!
I provide "primitives" for doing IO on files in a MemoryFileSystem. I delegate most of my actions to the MemoryFile. This way there is only one place needed where the data is stored.!

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:52'!
at: index
	^ entry at: index! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:52'!
at: index put: anObject
	^ entry at: index put: anObject! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:55'!
at: index read: aCollection startingAt: start count: count 
	^ entry at: index read: aCollection startingAt: start count: count ! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:55'!
at: first write: aCollection startingAt: start count: count 
	writable ifFalse: [ self primitiveFailed ].
	entry at: first write: aCollection startingAt: start count: count.! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/23/2012 20:06'!
close
	self isOpen ifFalse: [ ^ self ].
	self truncate.
	entry := nil.! !

!MemoryHandle methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/23/2012 19:57'!
copyFrom: from to: position
	^ entry copyFrom: from to: position! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:56'!
flush
	self truncate! !

!MemoryHandle methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 20:56'!
grow
	entry grow! !

!MemoryHandle methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/23/2012 18:47'!
grownBy: length
	entry grownBy: length! !

!MemoryHandle methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2012 20:57'!
isOpen
	^ entry notNil! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:57'!
open
	entry := self basicOpen.! !

!MemoryHandle methodsFor: 'stream-protocol' stamp: 'CamilloBruni 7/10/2012 14:56'!
readStream
	"Return a readstream on my contents.
	Using myself as target collection allows to share the internal bytearray between multiple streams."
	^ ReadStream on: self from: 1 to: entry fileSize! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/23/2012 18:48'!
size
	"return the size for the interna"
	^ entry internalSize! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:58'!
truncate
	entry truncate! !

!MemoryHandle methodsFor: 'public' stamp: 'CamilloBruni 6/23/2012 20:16'!
truncateTo: anInteger
	entry truncateTo: anInteger! !

!MemoryHandle methodsFor: 'stream-protocol' stamp: 'CamilloBruni 7/10/2012 14:54'!
writeStream
	"Return a writestream on my contents.
	Using myself as target collection allows to share the internal bytearray between multiple streams."
	^ WriteStream on: self from: 1 to: entry fileSize! !

Object subclass: #MemoryFileSystemEntry
	instanceVariableNames: 'creationTime modificationTime basename'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryFileSystemEntry commentStamp: '<historical>' prior: 0!
I am an abstract file system entry for a memory file system.
My subclasses should specialize on the kind of file they are.!

MemoryFileSystemEntry subclass: #MemoryFileSystemDirectory
	instanceVariableNames: 'entries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryFileSystemDirectory commentStamp: '<historical>' prior: 0!
I represent a memory file system entry for a directory!

!MemoryFileSystemDirectory methodsFor: 'creation' stamp: 'CamilloBruni 7/10/2012 14:53'!
ensureDirectory: aDirectoryName
	^ self 
		fileEntryAt: aDirectoryName
		put: (MemoryFileSystemDirectory named: aDirectoryName)! !

!MemoryFileSystemDirectory methodsFor: 'creation' stamp: 'CamilloBruni 7/10/2012 14:53'!
ensureFile: aFileName
	^ self 
		fileEntryAt: aFileName 
		put: (MemoryFileSystemFile named: aFileName)! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 20:02'!
entries
	^ entries! !

!MemoryFileSystemDirectory methodsFor: 'enumeration' stamp: 'CamilloBruni 7/10/2012 14:49'!
fileEntriesDo: aBlock
	
	entries keys sorted do: [ :fileName|
		aBlock value: (entries at: fileName)].! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:47'!
fileEntryAt: aFIleName
	^ entries at: aFIleName! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:47'!
fileEntryAt: aFileName ifAbsent: aBlock
	^ entries at: aFileName ifAbsent: aBlock! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:48'!
fileEntryAt: aFileName ifPresent: aBlock
	^ entries at: aFileName ifPresent: aBlock! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:48'!
fileEntryAt: aFIleName put: anEntry
	^ entries 
		at: aFIleName 
		ifAbsentPut: [
			self modified.
			anEntry ]! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:54'!
fileSize
	^ 0! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 20:17'!
includes: aFIleName
	^ entries includesKey: aFIleName! !

!MemoryFileSystemDirectory methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/22/2012 20:00'!
initialize
	super initialize.
	entries := Dictionary new.! !

!MemoryFileSystemDirectory methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2012 20:02'!
isDirectory
	^ true! !

!MemoryFileSystemDirectory methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 22:03'!
remove: aFileName
	modificationTime := DateAndTime now.
	^ entries removeKey: aFileName! !

!MemoryFileSystemEntry class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/10/2012 14:53'!
named: aFileName
	^ self new
		basename: aFileName;
		yourself! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:46'!
basename
	^ basename! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:46'!
basename: aString
	basename := aString! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 19:59'!
creationTime
	
	^ creationTime! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:54'!
fileSize
	self subclassResponsability ! !

!MemoryFileSystemEntry methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/22/2012 20:27'!
initialize 
	creationTime := modificationTime := DateAndTime now.
	super initialize! !

!MemoryFileSystemEntry methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2012 20:01'!
isDirectory
	self subclassResponsability! !

!MemoryFileSystemEntry methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2012 20:13'!
isFile
	^ self isDirectory not! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 19:59'!
modificationTime
	
	^ modificationTime! !

!MemoryFileSystemEntry methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 19:59'!
modificationTime: anObject
	
	modificationTime := anObject! !

!MemoryFileSystemEntry methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 22:05'!
modified
	modificationTime := DateAndTime now.! !

MemoryFileSystemEntry subclass: #MemoryFileSystemFile
	instanceVariableNames: 'bytes size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryFileSystemFile commentStamp: '<historical>' prior: 0!
I represent a memory file system entry for a regular file!

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 20:54'!
at: index
	self flag: #todo. "out of bounds checks?"
	^ bytes at: index! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 20:53'!
at: index put: anObject
	index > bytes size
		 ifTrue: [ self grow ].
		
	bytes at: index put: (anObject isCharacter
		ifTrue: [ anObject codePoint ]
		ifFalse: [ anObject ]).
	
	size := size max: index! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 20:54'!
at: index read: aCollection startingAt: start count: count 
	| max stop |
	max := size - index + 1 min: count.
	stop := start + max - 1.
	aCollection 
		replaceFrom: start
		to: stop
		with: bytes
		startingAt: index.
	^ stop - start + 1! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 21:25'!
at: first write: aCollection startingAt: start count: count 
	| last |
	last := first + count - 1.
	last > bytes size ifTrue: [ self grownBy: last - size ].
	bytes 
		replaceFrom: first
		to: last
		with: aCollection
		startingAt: start.
	size := last! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/23/2012 18:48'!
bytes
	^ bytes! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 22:15'!
copyFrom: from to: position
	^ bytes copyFrom: from to: position! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 7/10/2012 14:54'!
fileSize
	^ size! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/23/2012 19:58'!
grow
	self grownBy: self sizeIncrement! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 22:10'!
grownBy: length
	bytes := bytes grownBy: length.
	
	self modified.! !

!MemoryFileSystemFile methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/22/2012 20:53'!
initialize
	super initialize.
	bytes := #[].
	size := 0! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/23/2012 18:48'!
internalSize
	^ bytes size! !

!MemoryFileSystemFile methodsFor: 'testing' stamp: 'CamilloBruni 6/22/2012 20:02'!
isDirectory
	^ false! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 21:07'!
readStream
	^ ReadStream on: bytes from: 1 to: size! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/22/2012 20:57'!
sizeIncrement
	^ (bytes size min: 20) max: 1024! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/23/2012 20:16'!
truncate
	self truncateTo: size! !

!MemoryFileSystemFile methodsFor: 'accessing' stamp: 'CamilloBruni 6/23/2012 20:19'!
truncateTo: aSize
	bytes size = aSize 
		ifTrue: [ ^ self ].
	bytes := bytes copyFrom: 1 to: aSize.
	size := bytes size.
	
	self modified.! !

!MemoryFileSystemFile methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 22:01'!
updateSize: newSize
	size := newSize.
	modificationTime := nil.! !

!MemoryFileSystemFile methodsFor: 'stream-protocol' stamp: 'CamilloBruni 6/22/2012 22:16'!
writeStream 
	self flag: #todo. "need a nicer abstraction here to keep a shared bytes without calling grownBy on every at:put:"
	^ WriteStream on: self from: 1 to: size! !

FileSystemStore subclass: #MemoryStore
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FileSystem-Memory'!

!MemoryStore commentStamp: '<historical>' prior: 0!
I'm a specific store for memory file system!

!MemoryStore class methodsFor: 'public' stamp: 'cwp 2/18/2011 17:19'!
delimiter
	^ $/! !

!MemoryStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:20'!
isCaseSensitive
	^ true! !

!MemoryStore class methodsFor: 'public' stamp: 'CamilloBruni 5/7/2012 01:11'!
separator 
	^ $:! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 14:51'!
basenameFromEntry: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry basename! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 20:57'!
basicCreationTime: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry creationTime! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 20:30'!
basicIsDirectory: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry isDirectory! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 20:04'!
basicIsFile: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry isFile! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 20:57'!
basicModificationTime: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry modificationTime! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 21:06'!
basicOpen: path writable: aBoolean
	^ self
		nodeAt: path
		ifPresent: [ :aMemoryFileSystemEntry | aMemoryFileSystemEntry ]
		ifAbsent: [ 
			aBoolean
				ifFalse: [ self signalFileDoesNotExist: path ]
				ifTrue: [ self createFile: path ] ]! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 21:18'!
basicPosixPermissions: anEntry
	^ 8r777! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 21:13'!
basicSize: aMemoryFileSystemEntry
	^ aMemoryFileSystemEntry fileSize! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 5/10/2012 16:02'!
checkName: aString fixErrors: fixErrors
	aString ifEmpty: [ self error: 'zero length file name' ].
	^ aString! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 14:54'!
copy: sourcePath ifAbsent: absentBlock to: destinationPath ifPresent: presentBlock fileSystem: aFilesystem
	| sourceNode destinationNode |
	
	sourceNode := self
		nodeAt: sourcePath
		ifPresent: [ :source | source ]
		ifAbsent: [ ^ absentBlock value].
		
	sourceNode isDirectory 
		ifTrue: [ ^ absentBlock value ].
	
	destinationNode := self
		nodeAt: destinationPath parent
		ifPresent: [ :destination |  destination ]
		ifAbsent: [ ^ self signalDirectoryDoesNotExist: destinationPath parent ].
	
	destinationNode isFile
		ifTrue: [ self signalDirectoryDoesNotExist: destinationPath parent ].
		
	(destinationNode includes: destinationPath basename)
		ifTrue: [ "cannot overwrite existing file"^ presentBlock value ].
		
	destinationNode 
		fileEntryAt: destinationPath basename 
		put: sourceNode copy
! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 7/10/2012 14:53'!
createDirectory: path
	| parent |
	parent := path parent.
	^ self
		nodeAt: parent
		ifPresent: [ :entry | 
			entry
				fileEntryAt: path basename
				ifPresent: [ :node | 
					node isDirectory
						ifTrue: [ self signalDirectoryExists: path ]
						ifFalse: [ self signalFileExists: path ] ].
			entry ensureDirectory: path basename  ]
		ifAbsent: [ self signalDirectoryDoesNotExist: parent ]! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 6/22/2012 20:10'!
createFile: aPath
	^ self
		nodeAt: aPath parent
		ifPresent: [ :entry | 
			entry isDirectory
				ifTrue: [ entry ensureFile: aPath basename ]]
		ifAbsent: [ self signalDirectoryDoesNotExist: aPath parent ]! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 20:11'!
delete: path
	self 
		nodeAt: path parent
		ifPresent: [ :dict | dict remove: path basename ] 
		ifAbsent: [  ]! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 14:51'!
directoryAt: aPath ifAbsent: absentBlock nodesDo: aBlock
	^ self
		nodeAt: aPath
		ifPresent: [ :entry | 
			entry isDirectory
				ifFalse: [ ^ absentBlock value ].
			entry fileEntriesDo: aBlock ]
		ifAbsent: absentBlock! !

!MemoryStore methodsFor: 'printing' stamp: 'cwp 2/18/2011 16:32'!
forReferencePrintOn: aStream
	aStream nextPutAll: 'memory:'! !

!MemoryStore methodsFor: 'accessing' stamp: 'EstebanLorenzano 4/3/2012 09:36'!
handleClass
	^ MemoryHandle ! !

!MemoryStore methodsFor: 'initialize-release' stamp: 'CamilloBruni 6/22/2012 20:16'!
initialize 
	root := MemoryFileSystemDirectory new! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 14:52'!
nodeAt: aPath ifPresent: presentBlock ifAbsent: absentBlock
	| current |
	current := self root.
	aPath do: [ :segment | 
		current isDirectory
			ifTrue: [ current := current fileEntryAt: segment ifAbsent: [ ^ absentBlock value ]]
			ifFalse: [ ^ absentBlock value ]].
	^ presentBlock value: current! !

!MemoryStore methodsFor: 'public' stamp: 'CamilloBruni 6/22/2012 21:21'!
openFileStream: path writable: isWriteStream
	| entry |
	entry := self basicOpen: path writable: isWriteStream.
	^ isWriteStream 
		ifTrue: [ entry writeStream ]
		ifFalse: [ entry readStream ]! !

!MemoryStore methodsFor: 'private' stamp: 'CamilloBruni 7/10/2012 14:52'!
replaceFile: path in: aBlock
	^ self
		nodeAt: path parent
		ifPresent: [ :entry | | old new |
			entry isDirectory
				ifFalse: [ self signalFileDoesNotExist: path ].
			old := entry fileEntryAt: path basename ifAbsent: [ self signalFileDoesNotExist: path ].
			new := aBlock value: old.
			entry fileEntryAt: path basename put: new ]
		ifAbsent: [ self signalFileDoesNotExist: path ]! !

!MemoryStore methodsFor: 'accessing' stamp: 'cwp 2/18/2011 12:46'!
root
	^ root! !

!FileSystem class methodsFor: '*filesystem-memory' stamp: 'EstebanLorenzano 4/3/2012 13:13'!
inMemory
	"This method provides compatibility with the original Filesystem interface"
	self deprecated: 'Use #memory' on: '3 April 2012' in: 'Pharo 1.4'.
	^ self memory! !

!FileSystem class methodsFor: '*filesystem-memory' stamp: 'EstebanLorenzano 4/3/2012 09:36'!
memory
	^ self store: (MemoryStore new)! !
